SystemOrganization addCategory: #'System-Tools'!

Object subclass: #SpaceTally
	instanceVariableNames: 'results'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Tools'!

!SpaceTally commentStamp: 'sd 6/20/2003 22:31' prior: 0!
I'm responsible to help getting information about system space usage. The information I compute is represented by a spaceTallyItem

try something like: 

((SpaceTally new spaceTally: (Array with: TextMorph with: Point)) 
	asSortedCollection: [:a :b | a spaceForInstances > b spaceForInstances]) 

SpaceTally new systemWideSpaceTally


This class has been created from a part of SystemDictionary. It still deserves a nice
clean, such as using object instead of array having 4 slots.

sd-20 June 2003!

!SpaceTally methodsFor: 'class analysis' stamp: 'nice 1/5/2010 15:59'!
compareTallyIn: beforeFileName to: afterFileName
	"SpaceTally new compareTallyIn: 'tally' to: 'tally2'"

	| answer s beforeDict a afterDict allKeys |
	beforeDict := Dictionary new.
	s := FileDirectory default fileNamed: beforeFileName.
	[s atEnd] whileFalse: [
		a := Array readFrom: s nextLine.
		beforeDict at: a first put: a allButFirst.
	].
	s close.
	afterDict := Dictionary new.
	s := FileDirectory default fileNamed: afterFileName.
	[s atEnd] whileFalse: [
		a := Array readFrom: s nextLine.
		afterDict at: a first put: a allButFirst.
	].
	s close.
	answer := String new writeStream.
	allKeys := (Set new addAll: beforeDict keys; addAll: afterDict keys; yourself) asSortedCollection.
	allKeys do: [ :each | | before diff after |
		before := beforeDict at: each ifAbsent: [#(0 0 0)].
		after := afterDict at: each ifAbsent: [#(0 0 0)].
		diff := before with: after collect: [ :vBefore :vAfter | vAfter - vBefore].
		diff = #(0 0 0) ifFalse: [
			answer nextPutAll: each,'  ',diff printString; cr.
		].
	].
	StringHolder new contents: answer contents; openLabel: 'space diffs'.
	


! !

!SpaceTally methodsFor: 'class analysis' stamp: 'MarianoMartinezPeck 10/1/2010 16:24'!
computeSpaceUsage

	| entry c instanceCount |
	1 to: results size do: [:i |
		entry := results at: i.
		c := self class environment at: entry analyzedClassName.
		instanceCount := c instanceCount.
		entry codeSize: c spaceUsed.
		entry instanceCount: instanceCount.
		entry spaceForInstances: (c instancesSizeInMemory).
		Smalltalk garbageCollectMost].
	
! !

!SpaceTally methodsFor: 'class analysis' stamp: 'sd 6/20/2003 22:54'!
preAllocateResultsFor: classes

	results := OrderedCollection new: classes size.
	classes do: [:cl | results add: (SpaceTallyItem analyzedClassName: cl name)].
	results := results asArray.
! !

!SpaceTally methodsFor: 'fileout' stamp: 'stephane.ducasse 8/5/2009 17:07'!
printSpaceAnalysis	
	"SpaceTally new printSpaceAnalysis"

	| stream |
	stream := FileStream newFileNamed: 'STspace.text'.
	[ self printSpaceAnalysis: 1 on: stream ] ensure: [ stream close ]! !

!SpaceTally methodsFor: 'fileout' stamp: 'StephaneDucasse 9/7/2011 21:39'!
printSpaceAnalysis: threshold on: aStream
	"SpaceTally new printSpaceAnalysis: 1 on:(FileStream forceNewFileNamed: 'STspace.text')"

	"sd-This method should be rewrote to be more coherent within the rest of the class 
	ie using preAllocate and spaceForInstanceOf:"

	"If threshold > 0, then only those classes with more than that number
	of instances will be shown, and they will be sorted by total instance space.
	If threshold = 0, then all classes will appear, sorted by name."

	| totalCodeSpace totalInstCount totalInstSpace n totalPercent |
	
	Smalltalk garbageCollect.
	totalCodeSpace := totalInstCount := totalInstSpace := n := 0.
	results := OrderedCollection new: Smalltalk classNames size.
	'Taking statistics...'
		displayProgressFrom: 0 to: Smalltalk classNames size
		during: [:bar |
		(Smalltalk globals allClasses) do:
			[:cl | | codeSpace instCount instSpace eltSize | codeSpace := cl spaceUsed.
			bar value: (n := n+1).
			Smalltalk garbageCollectMost.
			instCount := cl instanceCount.
			instSpace := cl instancesSizeInMemory.
			totalCodeSpace := totalCodeSpace + codeSpace.
			totalInstCount := totalInstCount + instCount.
			totalInstSpace := totalInstSpace + instSpace.
			results add: (SpaceTallyItem analyzedClassName: cl name codeSize: codeSpace instanceCount:  instCount spaceForInstances: instSpace).]].
		totalPercent := 0.0.

	"aStream timeStamp."
	aStream
		nextPutAll: ('Class' padded: #right to: 45 with: $ );
		nextPutAll: ('code space' padded: #left to: 12 with: $ );
		nextPutAll: ('# instances' padded: #left to: 12 with: $ );
		nextPutAll: ('inst space' padded: #left to: 12 with: $ );
		nextPutAll: ('percent' padded: #left to: 12 with: $ ); 
		nextPutAll: ('inst average size' padded: #left to: 20 with: $ ); cr.

	threshold > 0 ifTrue: [
		"If inst count threshold > 0, then sort by space"
		results := (results select: [:s | s instanceCount >= threshold or: [s spaceForInstances > (totalInstSpace // 500)]])
			asSortedCollection: [:s :s2 | s spaceForInstances > s2 spaceForInstances]].

	results do: [:s | | percent |
		aStream
			nextPutAll: (s analyzedClassName padded: #right to: 45 with: $ );
			nextPutAll: (s codeSize printString padded: #left to: 12 with: $ );
			nextPutAll: (s instanceCount printString padded: #left to: 12 with: $ );
			nextPutAll: (s spaceForInstances printString padded: #left to: 14 with: $ ).
		percent := s spaceForInstances*100.0/totalInstSpace roundTo: 0.1.
		totalPercent := totalPercent + percent.
		aStream nextPutAll: ((percent printShowingDecimalPlaces: 2) padded: #left to: 12 with: $ ).
		aStream nextPutAll: (((s spaceForInstances / s instanceCount) asFloat printShowingDecimalPlaces: 2) padded: #left to: 20 with: $ ).
		aStream cr].
	
	aStream
		cr; nextPutAll: ('Total' padded: #right to: 45 with: $ );
		nextPutAll: (totalCodeSpace printString padded: #left to: 12 with: $ );
		nextPutAll: (totalInstCount printString padded: #left to: 12 with: $ );
		nextPutAll: (totalInstSpace printString padded: #left to: 14 with: $ );
		nextPutAll: ((totalPercent roundTo: 0.1) printString padded: #left to: 12 with: $ ).! !

!SpaceTally methodsFor: 'fileout' stamp: 'HenrikSperreJohansen 6/12/2010 02:43'!
printSpaceDifferenceFrom: fileName1 to: fileName2
	"For differential results, run printSpaceAnalysis twice with different fileNames,
	then run this method...
		SpaceTally new printSpaceAnalysis: 0 on: 'STspace.text1'.
			--- do something that uses space here ---
		SpaceTally new printSpaceAnalysis: 0 on: 'STspace.text2'.
		SpaceTally new printSpaceDifferenceFrom: 'STspace.text1' to: 'STspace.text2'
"
	| f coll1 coll2 item |
	f := FileStream readOnlyFileNamed: fileName1.
	coll1 := OrderedCollection new.
	[f atEnd] whileFalse: [coll1 add: f nextLine].
	f close.
	f := FileStream readOnlyFileNamed: fileName2.
	coll2 := OrderedCollection new.
	[f atEnd] whileFalse:
		[item := f nextLine.
		((coll1 includes: item) and: [(item endsWith: 'percent') not])
			ifTrue: [coll1 remove: item]
			ifFalse: [coll2 add: item]].
	f close.
	(StringHolder new contents: (String streamContents: 
			[:s | 
			s nextPutAll: fileName1; cr.
			coll1 do: [:x | s nextPutAll: x; cr].
			s cr; cr.
			s nextPutAll: fileName2; cr.
			coll2 do: [:x | s nextPutAll: x; cr]]))
		openLabel: 'Differential Space Analysis'.
! !

!SpaceTally methodsFor: 'accessing' stamp: 'sd 6/20/2003 22:31'!
results

	^ results! !

!SpaceTally methodsFor: 'fileout' stamp: 'sd 6/20/2003 22:59'!
saveTo: aFileName
	"| st |
	st := SpaceTally new.
	st spaceTally: (Array with: TextMorph with: Point).
	st saveTo: 'spaceTally2'"
	| s |
	(FileDirectory default fileExists: aFileName) ifTrue: [
		FileDirectory default deleteFileNamed: aFileName].
	s := FileDirectory default fileNamed: aFileName.
	results do: [:each | s nextPutAll: each analyzedClassName asString ; 
						nextPutAll: ' '; nextPutAll: each codeSize printString; 
						nextPutAll: ' '; nextPutAll: each instanceCount printString; 
						nextPutAll: ' '; nextPutAll: each spaceForInstances printString; cr].
	s close! !

!SpaceTally methodsFor: 'class analysis' stamp: 'sd 6/20/2003 22:24'!
spaceTally: classes
	"Answer a collection of SpaceTallyItems representing the memory space (in bytes) consumed 	by the code and instances of each class in the system. Note that code sizes do not currently 	report memory consumed by class variables. "

	"((SpaceTally new spaceTally: (Array with: TextMorph with: Point)) asSortedCollection: [:a :b | a spaceForInstances > b spaceForInstances]) asArray"

	self preAllocateResultsFor: classes.
	Smalltalk garbageCollect.
	self computeSpaceUsage.
	^ results
! !

!SpaceTally methodsFor: 'class analysis' stamp: 'StephaneDucasse 3/20/2010 14:29'!
systemWideSpaceTally
	"Answer a collection of SpaceTallyItems representing the memory space (in bytes) consumed 	by the code and instances of each class in the system. Note that code sizes do not currently 	report memory consumed by class variables. "

	"(SpaceTally new systemWideSpaceTally asSortedCollection: [:a :b | a spaceForInstances > b spaceForInstances]) asArray"

	^self spaceTally: Smalltalk globals allClasses.

! !

Object subclass: #SpaceTallyItem
	instanceVariableNames: 'analyzedClassName codeSize instanceCount spaceForInstances'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Tools'!

!SpaceTallyItem commentStamp: 'sd 6/20/2003 22:02' prior: 0!
I'm represent an entry in the spaceTally.!

!SpaceTallyItem class methodsFor: 'instance creation' stamp: 'sd 6/20/2003 22:54'!
analyzedClassName: aClassName

	^ self new
		analyzedClassName: aClassName ; yourself
		! !

!SpaceTallyItem class methodsFor: 'instance creation' stamp: 'sd 6/20/2003 22:54'!
analyzedClassName: aClassName codeSize: codeSize instanceCount: instanceCount spaceForInstances: spaceForInstances

	^ self new
		analyzedClassName: aClassName ;
		codeSize: codeSize ;
		instanceCount: instanceCount ;
		spaceForInstances: spaceForInstances ; yourself! !

!SpaceTallyItem methodsFor: 'accessing' stamp: 'sd 6/20/2003 22:59'!
analyzedClassName

	^ analyzedClassName! !

!SpaceTallyItem methodsFor: 'accessing' stamp: 'sd 6/20/2003 22:59'!
analyzedClassName: aClassName

	analyzedClassName := aClassName! !

!SpaceTallyItem methodsFor: 'accessing' stamp: 'sd 6/20/2003 22:08'!
codeSize

	^ codeSize! !

!SpaceTallyItem methodsFor: 'accessing' stamp: 'sd 6/20/2003 22:09'!
codeSize: aNumber

	codeSize := aNumber! !

!SpaceTallyItem methodsFor: 'accessing' stamp: 'sd 6/20/2003 22:09'!
instanceCount

	^ instanceCount! !

!SpaceTallyItem methodsFor: 'accessing' stamp: 'sd 6/20/2003 22:09'!
instanceCount: aNumber

	instanceCount := aNumber! !

!SpaceTallyItem methodsFor: 'printing' stamp: 'sd 6/20/2003 22:52'!
printOn: aStream

	analyzedClassName isNil
		ifFalse: [aStream nextPutAll: analyzedClassName asString]. 
	aStream nextPutAll: ' ('.
	codeSize isNil
		ifFalse: [aStream nextPutAll: 'code size: ' ;  nextPutAll: codeSize asString]. 
	instanceCount isNil
		ifFalse: [aStream nextPutAll: ' instance count: ' ;  nextPutAll: instanceCount asString]. 
	spaceForInstances isNil
		ifFalse: [aStream nextPutAll: ' space for instances: ' ;  nextPutAll: spaceForInstances asString]. 
	aStream nextPut: $).
	! !

!SpaceTallyItem methodsFor: 'accessing' stamp: 'sd 6/20/2003 22:10'!
spaceForInstances

	^ spaceForInstances! !

!SpaceTallyItem methodsFor: 'accessing' stamp: 'sd 6/20/2003 22:10'!
spaceForInstances: aNumber

	spaceForInstances := aNumber! !

Object subclass: #ToolRegistry
	instanceVariableNames: 'tools inspectorsMap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Tools'!

!ToolRegistry commentStamp: 'IgorStasenko 3/6/2011 18:03' prior: 0!
I am serving as a system-wide hub for various tools.
Instead of using direct class names for tools, which is not always nice because can introduce inter-package
dependencies, i provide a uniform way for accessing tools, via Smalltalk global:

Smalltalk tools someToolName,

where 'someToolName' is a name of the tool , under which some tool are registered.

Tools are adding themselves to registry by implementing #registerToolsOn: message at class side, for example:

MyClass>>registerToolsOn: registry
   registry register: self as: #myTool

Registers a MyClass as a tool under name #myTool, and can be accessed via:

Smalltalk tools myTool 

and avoid putting 'MyClass' reference into code.

You can add  ' self registerToolsOn: Smalltalk tools ' in class #initialize,
so your new tool(s) can be registered during package loading.

To reset tool registry to defaults , use: 
Smalltalk resetTools.

Also registry storing a map of inspector types. 
This is to replace an old scheme which using #inspectorClass , and introducing dependencies.
Now, an inspecting class dont needs to have any knowledge about its specialized inspector.
Instead, a specialized inspector could tell registry that it is available for inspecting instances of given class (see #registerInspector:for:  senders).
This allows to avoid dependency from instance class to its inspector class or using extension methods.


----------- 
The category 'menu' is for backward compatibility with ToolSet. We should remove it.

!

!ToolRegistry methodsFor: 'convenience-inspecting' stamp: 'IgorStasenko 10/29/2008 03:35'!
basicInspect: anObject
	"Open an inspector on the given object. The tool set must know which inspector type to use for which object - the object cannot possibly know what kind of inspectors the toolset provides."
	
	self using: #basicInspector do: [:tool | tool openOn: anObject	]! !

!ToolRegistry methodsFor: 'convenience-browsing' stamp: 'StephaneDucasse 10/15/2011 21:13'!
browse: aClass selector: aSelector
	"Open a browser"
	self deprecated: 'Use Smalltalk tools browser then a specific protocol like fullOnClass:selector:' on: '15 October 2011' in: 'Pharo1.4'.
	self using: #browser do: [:tool |
		tool fullOnClass: aClass selector: aSelector
	]! !

!ToolRegistry methodsFor: 'convenience-browsing' stamp: 'IgorStasenko 10/29/2008 03:07'!
browseChangeSetsWithClass: aClass selector: aSelector
	"Browse all the change sets with the given class/selector"
	self using: #changeSorter do: [:tool |
		^ tool browseChangeSetsWithClass: aClass selector: aSelector ]
! !

!ToolRegistry methodsFor: 'misc' stamp: 'IgorStasenko 3/6/2011 18:03'!
browseDirectToolReferences
	"Smalltalk tools browseDirectToolReferences"
	"Browse all direct references of all of my registered tools.
	Useful for converting calls like:

		MyTool foo
	into:
		Smalltalk tools myTool foo 	"
	
	| bindings sn |
	
	bindings := tools values select: [:each | each value isBehavior ]
		thenCollect: [:each | each value binding ].
	bindings := bindings copyWith: (Smalltalk bindingOf: #ToolSet).
	
	sn := SystemNavigation default.	

	^ sn
		browseMessageList: (
			sn allMethodsSelect: [:m |
				(m literals allButLast select: [:lit| lit isVariableBinding ]) includesAnyOf: bindings ] )
		name: 'All direct references to tools to fix'! !

!ToolRegistry methodsFor: 'convenience-browsing' stamp: 'IgorStasenko 10/29/2008 03:07'!
browseHierarchy: aClass selector: aSelector
	"Open a browser"

	(aClass == nil)  ifTrue: [^ self].
	
	self using: #browser do: [:tool | | newBrowser |
		(newBrowser := tool new) setClass: aClass selector: aSelector.
		newBrowser spawnHierarchy ]
! !

!ToolRegistry methodsFor: 'convenience-browsing' stamp: 'StephaneDucasse 9/17/2011 11:41'!
browseMessageList: messageList name: title autoSelect: autoSelectString
	"Open a message set browser"
	
	^ self using: #messageList do: [:tool |
		tool 
			openMessageList: messageList 
			name: title 
			autoSelect: autoSelectString ]	! !

!ToolRegistry methodsFor: 'convenience-browsing' stamp: 'IgorStasenko 10/29/2008 03:07'!
browseMessageNames: aString

	self using: #messageNames do: [:tool | 
		^ (tool methodBrowserSearchingFor: aString) openInWorld
		]
! !

!ToolRegistry methodsFor: 'convenience-browsing' stamp: 'StephaneDucasse 9/1/2011 15:58'!
browseSendersOf: aSymbol name: titleString autoSelect: autoSelectString

	^self 
		browseMessageList: (SystemNavigation default allCallsOn: aSymbol) asSortedCollection
		name: titleString 
		autoSelect: autoSelectString! !

!ToolRegistry methodsFor: 'convenience-browsing' stamp: 'nice 10/31/2011 23:25'!
browseVersionsOf: aClass selector: aSelector
	"Open a browser"
	
	self using: #versionBrowser do: [:tool | 
		tool
			browseVersionsOf: (aClass compiledMethodAt: aSelector)
			class: aClass theNonMetaClass
			meta: aClass isMeta
			category: (aClass organization categoryOfElement: aSelector)
			selector: aSelector ]! !

!ToolRegistry methodsFor: 'misc' stamp: 'MarcusDenker 5/27/2011 14:24'!
codeCompletionAround: aBlock textMorph: aTextMorph keyStroke: evt
	^(self hasToolNamed: #codeCompletion)
	ifTrue: [ self codeCompletion codeCompletionAround: aBlock textMorph: aTextMorph keyStroke: evt ]
	ifFalse: [ aBlock value ]! !

!ToolRegistry methodsFor: 'convenience-debugging' stamp: 'IgorStasenko 10/29/2008 03:09'!
debug: aProcess context: aContext label: aString contents: contents fullView: aBool
	"Open a debugger on the given process and context."
	
	self using: #debugger do: [:tool |
		^ tool 
			openOn: aProcess 
			context: aContext 
			label: aString 
			contents: contents 
			fullView: aBool ]! !

!ToolRegistry methodsFor: 'convenience-debugging' stamp: 'IgorStasenko 10/29/2008 03:08'!
debugContext: aContext label: aString contents: contents
	"Open a debugger on the given process and context."
	self using: #debugger do: [:tool |
		^ tool openContext: aContext label: aString contents: contents
		]! !

!ToolRegistry methodsFor: 'convenience-debugging' stamp: 'IgorStasenko 10/29/2008 03:02'!
debugError: anError
	"Handle an otherwise unhandled error"
	^Processor activeProcess
		debug: anError signalerContext
		title: anError description! !

!ToolRegistry methodsFor: 'convenience-debugging' stamp: 'IgorStasenko 10/29/2008 03:08'!
debugSyntaxError: anError
	"Handle a syntax error.
	
	Note, if there is no #syntaxError registered, it will be silently ignored"
	
	self using: #syntaxError do: [:tool | | notifier |
		notifier := tool new
			setClass: anError errorClass
			code: anError errorCode
			debugger: (self debugger context: anError signalerContext)
			doitFlag: anError doitFlag.
			notifier category: anError category.
			tool open: notifier.
		]! !

!ToolRegistry methodsFor: 'handling DNU' stamp: 'IgorStasenko 2/19/2011 03:15'!
doesNotUnderstand: aMessage
	"Return a tool identified by a message selector.
	We need to handle special case, when no debugger registered.
	"
	| tool |
	tool := tools at: aMessage selector ifAbsent: nil.
	tool ifNotNil: [ ^ tool value ].

	^ (aMessage selector == #debugger) 
		ifTrue: [ 
			Smalltalk 
				logError: 'No debugger installed to handle debug request.' 
				inContext: thisContext.
		
			nil. "Smalltalk snapshot: false andQuit: true."
		]
		ifFalse: [ super doesNotUnderstand: aMessage ]
! !

!ToolRegistry methodsFor: 'convenience-inspecting' stamp: 'IgorStasenko 10/29/2008 03:36'!
explore: anObject
	"Open an explorer on the given object."
	
	self using: #objectExplorer do: [:tool | tool new openExplorerFor: anObject	]
! !

!ToolRegistry methodsFor: 'convenience-inspecting' stamp: 'IgorStasenko 2/20/2011 15:41'!
exploreViewHierarchy: aMorph
	"Open an explorer on the given object."
	^ self viewHierarchyExplorer openOn: aMorph! !

!ToolRegistry methodsFor: 'testing' stamp: 'IgorStasenko 2/19/2011 03:18'!
hasToolNamed: aToolName
	^ tools includesKey: aToolName ! !

!ToolRegistry methodsFor: 'misc' stamp: 'IgorStasenko 10/29/2008 02:09'!
informNoDebugger
	(self includes: #uiManager) 
		ifTrue: [
			self uiManager inform: 'Cannot open debugger' ]
		ifFalse: [
			"If we throw an exception here, this can lead to recursion.
			Any idea what else we can do here?"
			Processor activeProcess terminate ]! !

!ToolRegistry methodsFor: 'initialize-release' stamp: 'IgorStasenko 8/12/2011 16:37'!
initDefaultToolSet
	
	"Query the system for registering tools"
	"we cannot afford to show warnings because not all tools e.g Debugger, Inspector are registered yet"
	
	[	
	Object withAllSubclassesDo: [:ea | 
		
	"evaluate only for classes which directly implement this message,
	not via subclassing"
		(ea isMeta  and: [
			ea includesSelector: #registerToolsOn: ]) ifTrue: [
			 ea theNonMetaClass registerToolsOn: self  ]].
	] on: Warning do: [:ex | ex resume ]! !

!ToolRegistry methodsFor: 'initialize-release' stamp: 'IgorStasenko 2/20/2011 14:58'!
initialize

	"symbols used as keys "
	tools := IdentityDictionary new.  
	inspectorsMap := IdentityDictionary new.
	
	self initDefaultToolSet! !

!ToolRegistry methodsFor: 'convenience-inspecting' stamp: 'IgorStasenko 2/20/2011 15:03'!
inspect: anObject
	"Open an inspector on the given object. 
	
	You can register custom inspector for own class.
	Use:
	
	Smalltalk tools registerInspector: #MyaIspectorClass for: #MyInstancesClass "
	
	| class |
	
	class := (self inspectorClassOf: anObject)
		ifNil: [ tools at: #inspector ifAbsent: [ self error: 'no inspector found' ]].
	
	class ifNotNil: [ ^ class openOn: anObject ]
	
! !

!ToolRegistry methodsFor: 'convenience-inspecting' stamp: 'IgorStasenko 2/20/2011 15:03'!
inspect: anObject label: aString
	"Open an inspector on the given object. 
	
	You can register custom inspector for own class.
	Use:
	
	Smalltalk tools registerInspector: #MyaIspectorClass for: #MyInstancesClass 
	"
	| class |
	
	class := self inspectorClassOf: anObject. 
	class ifNil: [ tools at: #inspector ifAbsent: []].
	
	class ifNotNil: [ ^ class openOn: anObject withEvalPane: true withLabel: aString ]
! !

!ToolRegistry methodsFor: 'convenience-inspecting' stamp: 'IgorStasenko 2/20/2011 15:02'!
inspectorClassOf: anObject
	"Answer the inspector class for the given object. The tool set must know which inspector type to use for which object - the object cannot possibly know what kind of inspectors the toolset provides."
		
	anObject class withAllSuperclassesDo: [:cls | 
		inspectorsMap at: cls name ifPresent: [:inspectorName |
			^ Smalltalk classNamed: inspectorName
		]].

	^ nil
	
! !

!ToolRegistry methodsFor: 'convenience-debugging' stamp: 'IgorStasenko 10/29/2008 03:10'!
interrupt: aProcess label: aString
	"Open a debugger on the given process and context."
	
	self using: #debugger do: [:tool | 
		tool openInterrupt: aString onProcess: aProcess
		]
! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/19/2011 03:26'!
mainMenuItems
	"Answer the most important menu items available for this tool set"
	^#(
		('System Browser' 			#openClassBrowser)
		('Workspace'				#openWorkspace)
		('Test Runner'				#openTestRunner)
		('Monticello Browser'		#openMonticelloBrowser)
	)
! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/20/2011 15:32'!
menuItems
	"Answer the menu items available for this tool set"

	^#(
		('System Browser' 			#openClassBrowser)
		('Message Names'			#openMessageNames)
		-
		('Workspace'				#openWorkspace)
		('Transcript' 				#openTranscript)
		('File Browser'				#openFileList)
		-
		('Test Runner'				#openTestRunner)
		('Process Browser' 			#openProcessBrowser)
		-
		('Monticello Browser'		#openMonticelloBrowser)
		"('Monticello Configurations' #openMonticelloConfigurations)"
		('Recover lost changes...'	#openRecentChangesLog)
		"('Simple Change Sorter'		#openChangeSorter)"
		('Change Sorter'				#openDualChangeSorter)
	)
! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/19/2011 03:57'!
openChangeSorter
	^ self changeSorter open
! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/19/2011 03:55'!
openClassBrowser
	^ self browser open! !

!ToolRegistry methodsFor: 'convenience-browsing' stamp: 'IgorStasenko 10/29/2008 03:07'!
openClassListBrowser: anArray title: aString
	"Open a class list browser"
	self using: #classListBrowser do: [:tool |
		^ tool new initForClassesNamed: anArray title: aString
		]
! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/20/2011 14:42'!
openDualChangeSorter
	self dualChangeSorter open! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/20/2011 14:38'!
openFileList
	
	self fileList open! !

!ToolRegistry methodsFor: 'menu' stamp: 'BernardoContreras 8/14/2011 02:23'!
openFinder
	^ self finder open! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 10/29/2008 03:42'!
openMessageNames
	"Bring a MessageNames tool to the front"
	
	self using: #messageNames do: [:tool | tool openMessageNames ]
! !

!ToolRegistry methodsFor: 'menu' stamp: 'sd 6/5/2011 19:05'!
openMonticelloBrowser
	
	self monticelloBrowser open
	! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 10/29/2008 03:43'!
openProcessBrowser
	self using: #processBrowser do: [:tool |
		tool open
		]
! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/20/2011 14:39'!
openRecentChangesLog
	self changeList browseRecentLog! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 10/29/2008 03:46'!
openSelectorBrowser
	self using: #selectorBrowser do: [:tool |
		tool new morphicWindow openInWorld
	]
! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/19/2011 04:02'!
openTestRunner
	self using: #testRunner do: [:tool | tool open ]! !

!ToolRegistry methodsFor: 'menu' stamp: 'FernandoOlivero 5/9/2011 09:38'!
openTranscript

	^ UIManager default openTranscript! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/19/2011 03:58'!
openWorkspace
	^ self workspace open! !

!ToolRegistry methodsFor: 'registration' stamp: 'IgorStasenko 2/7/2011 18:49'!
register: anObject as: aNameSelector
	"Register a tool class under given name"
	tools at: aNameSelector put: anObject! !

!ToolRegistry methodsFor: 'convenience-inspecting' stamp: 'IgorStasenko 2/20/2011 14:58'!
registerInspector: anInspectorClass for: anInstanceClass

	"Register inspector class for given instance class" 

	inspectorsMap at: anInstanceClass name put: anInspectorClass name
! !

!ToolRegistry methodsFor: 'registration' stamp: 'IgorStasenko 10/29/2008 01:31'!
remove: aName
	"remove tool with given name"
	tools removeKey: aName ifAbsent: []! !

!ToolRegistry methodsFor: 'tools' stamp: 'IgorStasenko 2/20/2011 14:36'!
transcript
	"No need to register transcript. But for uniformity,
	implement this method, so transcript can be accessed via
	Smalltalk tools transcript , as any other tool "
	^ Transcript! !

!ToolRegistry methodsFor: 'misc' stamp: 'IgorStasenko 2/19/2011 04:03'!
using: aToolName do: aBlock
	"Evaluate a block for tool found under corresponding name or silently ignore if its not found or nil"
	| tool |
	
	tool := tools at: aToolName ifAbsent: [ ^ self ].
	tool ifNotNil: [ ^ aBlock value: tool value ]! !

!SmalltalkImage methodsFor: '*System-Tools' stamp: 'IgorStasenko 8/17/2011 16:27'!
reportCPUandRAM
	"Write several text files with useful analysis for profiling purposes.
	Overwrites any existing report.
	SmalltalkImage current reportCPUandRAM
	"	

	| stream |
	"VM statistics (Memory use and GC, mainly)"
	FileStream 
		forceNewFileNamed: 'Log-MemoryStats.txt'
		do: [:aStream | aStream nextPutAll: self vm statisticsReport ].
	
	"Process list"
	stream := FileStream 
				forceNewFileNamed: 'Log-ProcessList.txt' 
				do: [ :aStream |  self tools processBrowser new processNameList 
								do: [ :each | aStream nextPutAll: each; cr ]].

"Fork all these, so they run in sequence, as the system is back running"
[
	"Process taking most CPU"
	stream := FileStream forceNewFileNamed: 'Log-ThePig.txt'.
	self tools processBrowser dumpPigStackOn: stream andClose: true.
	
	"Tally of all processes"
	stream := FileStream forceNewFileNamed: 'Log-FullTally.txt'.
	[ | tally |
		tally := MessageTally new.
		tally reportOtherProcesses: true.
		tally spyAllEvery: 1 on: [ (Delay forMilliseconds: 1000) wait ].
		tally report: stream ] ensure: [ stream close ].

	"Memory Analysis"
	stream := FileStream forceNewFileNamed: 'Log-MemoryAnalysis.txt'.
	[ SpaceTally new printSpaceAnalysis: 1 on: stream ]
		ensure: [ stream close ]

] forkNamed: 'CPU usage analysis'! !

!SmalltalkImage methodsFor: '*System-Tools' stamp: ''!
textMarkerForShortReport

	^  'Since last view	'! !
